# This file was auto-generated by Fern from our API Definition.

import typing

from .error_vellum_value import ErrorVellumValue
from .function_call_vellum_value import FunctionCallVellumValue
from .json_vellum_value import JsonVellumValue
from .string_vellum_value import StringVellumValue
from .thinking_vellum_value import ThinkingVellumValue

PromptOutput = typing.Union[
    StringVellumValue, JsonVellumValue, ErrorVellumValue, FunctionCallVellumValue, ThinkingVellumValue
]
