# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .workflow_node_result_event_state import WorkflowNodeResultEventState


class NodeOutputCompiledArrayValue(UniversalBaseModel):
    """
    An output returned by a node that is of type ARRAY.
    """

    type: typing.Literal["ARRAY"] = "ARRAY"
    value: typing.Optional[typing.List["VellumValue"]] = None
    node_output_id: str
    state: typing.Optional[WorkflowNodeResultEventState] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .array_vellum_value import ArrayVellumValue  # noqa: E402, F401, I001
from .vellum_value import VellumValue  # noqa: E402, F401, I001

update_forward_refs(NodeOutputCompiledArrayValue)
