# This file was auto-generated by Fern from our API Definition.

import typing

IntegrationName = typing.Union[
    typing.Literal[
        "SLACK",
        "NOTION",
        "GOOGLE",
        "CALENDLY",
        "CANVA",
        "CLICKUP",
        "CODA",
        "HUBSPOT",
        "INTERCOM",
        "LINEAR",
        "LINKUP",
        "LISTENNOTES",
        "LMNT",
        "LINKEDIN",
        "MAILCHIMP",
        "MEM0",
        "NEON",
        "GITHUB",
        "GOOGLE_SHEETS",
        "GOOGLE_CALENDAR",
        "GOOGLE_DRIVE",
        "GMAIL",
        "ACCULYNX",
        "AFFINITY",
        "AGENCYZOOM",
        "AHREFS",
        "AIRTABLE",
        "APOLLO",
        "ASANA",
        "ATLASSIAN",
        "BITBUCKET",
        "BOX",
        "BREVO",
        "BROWSERBASE_TOOL",
        "CAL",
        "CANVAS",
        "ELEVENLABS",
        "EXA",
        "GAMMA",
        "GITLAB",
        "FIRECRAWL",
        "FIGMA",
        "FIREFLIES",
        "GOOGLE_MAPS",
        "HEYGEN",
        "JIRA",
        "JUNGLESCOUT",
        "KLAVIYO",
        "PAGERDUTY",
        "PARSERA",
        "PEOPLEDATALABS",
        "PERPLEXITY",
        "POSTHOG",
        "PRODUCTBOARD",
        "REDDIT",
        "SEMRUSH",
        "SEMANTICSCHOLAR",
        "SENDGRID",
        "SERPAPI",
        "SHARE_POINT",
        "SHORTCUT",
        "STRIPE",
        "SUPABASE",
        "TAVILY",
        "TELEGRAM",
        "WEBFLOW",
        "YOUSEARCH",
        "ZENDESK",
        "ZENROWS",
        "DROPBOX",
        "EVENTBRITE",
        "CONFLUENCE",
        "COINBASE",
        "DISCORD",
        "DOCUSIGN",
        "TRELLO",
    ],
    typing.Any,
]
