# This file was auto-generated by Fern from our API Definition.

from .pdf_search_result_meta_source import PdfSearchResultMetaSource

ComponentsSchemasPdfSearchResultMetaSource = PdfSearchResultMetaSource
