# This file was auto-generated by Fern from our API Definition.

import typing

from .prompt_request_input import PromptRequestInput
from .workflow_input import WorkflowInput

DeprecatedPromptRequestInput = typing.Union[WorkflowInput, PromptRequestInput]
