# This file was auto-generated by Fern from our API Definition.

import typing

from .plain_text_prompt_block import PlainTextPromptBlock
from .variable_prompt_block import VariablePromptBlock

RichTextChildBlock = typing.Union[VariablePromptBlock, PlainTextPromptBlock]
