# This file was auto-generated by Fern from our API Definition.

import typing

SecretTypeEnum = typing.Union[
    typing.Literal["USER_DEFINED", "HMAC", "INTERNAL_API_KEY", "EXTERNALLY_PROVISIONED"], typing.Any
]
