# This file was auto-generated by Fern from our API Definition.

import typing

from .workflow_event_error import WorkflowEventError
from .workflow_initialization_error import WorkflowInitializationError

WorkflowError = typing.Union[WorkflowEventError, WorkflowInitializationError]
