# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .prompt_node_execution_meta import PromptNodeExecutionMeta


class PromptNodeResultData(UniversalBaseModel):
    execution_meta: typing.Optional[PromptNodeExecutionMeta] = None
    output_id: str
    array_output_id: typing.Optional[str] = None
    execution_id: typing.Optional[str] = None
    text: typing.Optional[str] = None
    delta: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
