# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing
from pydantic import Field

if typing.TYPE_CHECKING:
    from .api_request_parent_context import ApiRequestParentContext
    from .external_parent_context import ExternalParentContext
    from .integration_trigger_context import IntegrationTriggerContext
    from .node_parent_context import NodeParentContext
    from .prompt_deployment_parent_context import PromptDeploymentParentContext
    from .scheduled_trigger_context import ScheduledTriggerContext
    from .workflow_deployment_parent_context import WorkflowDeploymentParentContext
    from .workflow_parent_context import WorkflowParentContext
    from .workflow_sandbox_parent_context import WorkflowSandboxParentContext
ParentContext = typing.Annotated[
    typing.Union[
        "WorkflowParentContext",
        "NodeParentContext",
        "WorkflowDeploymentParentContext",
        "WorkflowSandboxParentContext",
        "PromptDeploymentParentContext",
        "ApiRequestParentContext",
        "ExternalParentContext",
        "ScheduledTriggerContext",
        "IntegrationTriggerContext",
    ],
    Field(discriminator="type")
]
