# This file was auto-generated by Fern from our API Definition.

import typing

from .folder_entity_dataset import FolderEntityDataset
from .folder_entity_document_index import FolderEntityDocumentIndex
from .folder_entity_folder import FolderEntityFolder
from .folder_entity_prompt_sandbox import FolderEntityPromptSandbox
from .folder_entity_test_suite import FolderEntityTestSuite
from .folder_entity_workflow_sandbox import FolderEntityWorkflowSandbox

FolderEntity = typing.Union[
    FolderEntityFolder,
    FolderEntityPromptSandbox,
    FolderEntityWorkflowSandbox,
    FolderEntityDocumentIndex,
    FolderEntityTestSuite,
    FolderEntityDataset,
]
