# This file was auto-generated by Fern from our API Definition.

import typing

NewMemberJoinBehaviorEnum = typing.Union[
    typing.Literal["AUTO_ACCEPT_FROM_SHARED_DOMAIN", "ALLOW_REQUESTS_FROM_SHARED_DOMAIN", "REQUIRE_EXPLICIT_INVITE"],
    typing.Any,
]
