# This file was auto-generated by Fern from our API Definition.

import typing

ApiActorTypeEnum = typing.Union[
    typing.Literal["WORKSPACE_API_KEY", "ENVIRONMENT_API_KEY", "JWT", "SERVICE_TOKEN"], typing.Any
]
