# This file was auto-generated by Fern from our API Definition.

import typing

from .workflow_request_audio_input_request import WorkflowRequestAudioInputRequest
from .workflow_request_chat_history_input_request import WorkflowRequestChatHistoryInputRequest
from .workflow_request_document_input_request import WorkflowRequestDocumentInputRequest
from .workflow_request_image_input_request import WorkflowRequestImageInputRequest
from .workflow_request_json_input_request import WorkflowRequestJsonInputRequest
from .workflow_request_number_input_request import WorkflowRequestNumberInputRequest
from .workflow_request_string_input_request import WorkflowRequestStringInputRequest
from .workflow_request_video_input_request import WorkflowRequestVideoInputRequest

WorkflowRequestInputRequest = typing.Union[
    WorkflowRequestStringInputRequest,
    WorkflowRequestJsonInputRequest,
    WorkflowRequestChatHistoryInputRequest,
    WorkflowRequestNumberInputRequest,
    WorkflowRequestAudioInputRequest,
    WorkflowRequestVideoInputRequest,
    WorkflowRequestImageInputRequest,
    WorkflowRequestDocumentInputRequest,
]
